@@ -20,33 +20,22 @@ setMethod(".gradio",
 
             force(toolkit)
 
-            if(is.data.frame(items))
-              items <- items[,1, drop=TRUE] # first column
-            
-            if (length(items)<2)
-              stop("Radio button group makes sense only with at least two items.")
-            
             if(horizontal)
               g <- gtkHBox()
             else
               g <- gtkVBox()
 
             radiogp <- gtkRadioButton(group=NULL, label=items[1]) # initial
-            sapply(2:length(items),function(i)
-                   radiogp$NewWithLabelFromWidget(items[i]))
-
-            ## when using GetGroup, this gives reverse order
-            ## efficiency of gtk list
-            
-
-            ## layout
-            sapply(rev(radiogp$GetGroup()),         # reverse list
-                   function(i) g$PackStart(i))
 
             obj <- as.gWidgetsRGtk2(radiogp, block=g)
 
+            if(is.data.frame(items))
+              items <- items[,1, drop=TRUE] # first column
+
+            obj[] <- items
             svalue(obj,index=TRUE) <- selected
-            
+
+            tag(obj, ".handlers") <- list() # list of handlers keyed by ID
             
             ## do we add to the container?
             if (!is.null(container)) {
@@ -57,18 +46,19 @@ setMethod(".gradio",
   
             ## add handler
             if(!is.null(handler))
-              addhandlerchanged(obj, handler, action)
+              addHandlerChanged(obj,  handler, action)
 
             
             invisible(obj)
           })
 
+##' coercion method from a gtkRadioButton widget. Pass in container via bloc
 as.gWidgetsRGtk2.GtkRadioButton <- function(widget,...) {
   theArgs <- list(...)
   if(!is.null(theArgs$block))
     block <- theArgs$block
   else
-    block <- widget
+    block <- gtkHBox()                  # or vbox!
 
   obj <- new("gRadioRGtk",block=block, widget=widget,
     toolkit=guiToolkit("RGtk2"))
@@ -108,13 +98,13 @@ setReplaceMethod(".svalue",
                      if(value %in% 1:length(obj))
                        btns[[as.numeric(value)]]$SetActive(TRUE)
                      else
-                       cat(sprintf("index outside of range"))
+                       cat(sprintf("index outside of range\n"))
                    } else {
                      if(value %in% items) {
                        whichIndex = min(which(value == items))
                        btns[[whichIndex]]$SetActive(TRUE)
                      } else {
-                       cat(sprintf("Value %s is not among the items",value))
+                       cat(sprintf("Value %s is not among the items\n",value))
                      }
                    }
                    
@@ -128,6 +118,7 @@ setMethod(".leftBracket",
           function(x, toolkit, i, j, ..., drop=TRUE) {
             radiogp <- getWidget(x)
             btns <- rev(radiogp$GetGroup())
+            btns <- btns[1:tag(x,".n")]
             items <- sapply(btns, function(i) i$GetLabel())
             
             if(missing(i))
@@ -149,33 +140,63 @@ setReplaceMethod(".leftBracket",
             radiogp <- getWidget(x)
             gp <- getBlock(x)
             btns <- rev(radiogp$GetGroup())
-            items <- x[]
             n = length(x)
             
             ## set items
             if(missing(i)) {
-              ## can't reduce size, just lengthen
-              if(length(value) < length(x)) {
-                cat(sprintf("Can't reduce length of radio button group"))
+
+              ## The radio group doesn't like to reduce the size. We trick it by
+              ## keeping track of a variable length in ".n" tag
+              n <- length(value)
+              if(n < 2) {
+                cat(sprintf("Length of items must be 2 or more\n"))
                 return(x)
               }
 
-              if(n < length(value)) {
-                ## add some new buttons; also pack in
-                for(i in 1:(length(value)-n)) {
-                  b <- radiogp$NewWithLabelFromWidget("")
-                  if(inherits(gp,"GtkBox"))
-                    gp$PackStart(b)
-                  else
-                    cat(sprintf("can't pack in object of class %s",class(gp)))
-                }
-              }
-              ## replace all the values
-              btns <- rev(radiogp$GetGroup()) # maybe grew in length
-              sapply(1:length(btns), function(i)
-                     btns[[i]]$SetLabel(value[i]))
+              ## we store the length of items in the .n value.
+              ## When shortening a lenght by setting the group, the GTK
+              ## widget does not truncate. We use this to do so. (leaving some
+              ## possible orphans in the radioButtonGroup object.)
+              tag(x, ".n") <- n
+
+              ## clear old
+              sapply(gp$getChildren(), function(i) gp$remove(i))
+              ## make new
+              radiogp1 <- gtkRadioButton(group=NULL, label=value[1])
+              sapply(value[-1], function(i) {
+                radiogp1$newWithLabelFromWidget(i)
+              })
+              ## replace -- doesn't clear, just replaces first n (even if more than n)
+              radiogp$setGroup(radiogp1$getGroup())
+              
+              ## now add to container
+              btns <- rev(radiogp$getGroup())[1:n] # no more than n of them
+              sapply(btns, function(i) gp$PackStart(i))
+
+              ## need to add in the handlers
+              ## Always call to see if a handler exists
+              sapply(btns, function(i) {
+                gSignalConnect(i, "toggled", f=function(obj, w, ...) {
+                  if(w$getActive()) {
+                    ## call handlers from h
+                    handlers <- tag(obj, ".handlers")
+                    if(length(handlers)) {
+                      ## handler is list with blocked, handler, action component
+                      sapply(handlers, function(handler) {
+                        if(!handler$blocked)
+                          handler$handler(list(obj=obj, action=handler$action), ...)
+                      })
+                    }
+                  }
+                },
+                               data=x,
+                               user.data.first=TRUE,
+                               after=FALSE
+                               )
+              })
             } else {
               ## update just the i values
+              i <- i[i <= n]
               for(j in 1:length(i)) 
                 btns[[j]]$SetLabel(value[j])
             }
@@ -195,9 +216,10 @@ setReplaceMethod("[",
 setMethod(".length",
           signature(toolkit="guiWidgetsToolkitRGtk2",x="gRadioRGtk"),
           function(x,toolkit) {
-            radiogp <- getWidget(x)
-            btns <- rev(radiogp$GetGroup())
-            length(btns)
+            tag(x, ".n")
+#            radiogp <- getWidget(x)
+#            btns <- rev(radiogp$GetGroup())
+#            length(btns)
           })
 
 
@@ -231,41 +253,113 @@ setReplaceMethod(".visible",
 
 ##################################################
 ## handlers
+
+## need to deal with changing buttons via [<-
+## added a handlers cache that we can manipulate
 setMethod(".addhandlerclicked",
           signature(toolkit="guiWidgetsToolkitRGtk2",obj="gRadioRGtk"),
           function(obj, toolkit, handler, action=NULL, ...) {
-            .addhandlerchanged(obj,toolkit,handler,action,...)
+
+            handlers <- tag(obj, ".handlers")
+            if(length(handlers))
+              nhandlers <- max(as.numeric(names(handlers)))
+            else
+              nhandlers <- 0
+
+            newhandler <- list(blocked=FALSE,
+                               handler=handler,
+                               action=action)
+            ID <- as.character(nhandlers + 1)
+            handlers[[ID]] <- newhandler
+            tag(obj, ".handlers") <- handlers
+
+            invisible(ID)
+            
+ 
           })
+ 
+
+
 
 setMethod(".addhandlerchanged",
           signature(toolkit="guiWidgetsToolkitRGtk2",obj="gRadioRGtk"),
           function(obj, toolkit, handler, action=NULL, ...) {
-
-            radiogp <- getWidget(obj)
-            btns <- rev(radiogp$GetGroup())
-            
-            IDs = sapply(btns, function(x) {
-              gtktry(connectSignal(x,
-                                   signal="toggled",
-                                   f=function(h,w,...) {
-                                     ## only call handler for change to active
-                                     ## not just toggle
-                                     if(w$GetActive())
-                                       handler(h,w,...)
-                                   },
-                                   data=list(obj=obj, action=action,...),
-                                   user.data.first = TRUE,
-                                   after = FALSE), silent=FALSE)
-            })
-            
-            handler.ID = tag(obj, "handler.id")
-            if(is.null(handler.ID))
-              handler.ID =list()
-            for(i in 1:length(IDs))
-              handler.ID[[length(handler.ID)+1]] = IDs[[i]]
-            tag(obj, "handler.id", replace=FALSE) <- handler.ID
-            
-            invisible(IDs)
+            .addhandlerclicked(obj,toolkit,handler,action,...)
           })
+
+
+
+setMethod(".removehandler",
+          signature(toolkit="guiWidgetsToolkitRGtk2",obj="gRadioRGtk"),
+          function(obj, toolkit, ID=NULL, ...) {
+            handlers <- tag(obj, ".handlers")
+            
+            if(is.null(ID)) {
+              handlers <- list()        # remove all
+            } else {
+              sapply(ID, function(id) {
+                handlers[[id]] <<- NULL
+              })
+            }
+            tag(obj, ".handlers") <- handlers          
+          })
+
+setMethod(".blockhandler",
+          signature(toolkit="guiWidgetsToolkitRGtk2",obj="gRadioRGtk"),
+          function(obj, toolkit, ID=NULL, ...) {
+            handlers <- tag(obj, ".handlers")
+            if(is.null(ID)) {
+              ID <- names(handlers)
+            }
+            sapply(ID, function(id) {
+              handlers[[id]]$blocked <<- TRUE
+            })
+            tag(obj, ".handlers") <- handlers          
+          })
+
+setMethod(".unblockhandler",
+          signature(toolkit="guiWidgetsToolkitRGtk2",obj="gRadioRGtk"),
+          function(obj, toolkit, ID=NULL, ...) {
+            handlers <- tag(obj, ".handlers")
+            if(is.null(ID)) {
+              ID <- names(handlers)
+            }
+            sapply(ID, function(id) {
+              handlers[[id]]$blocked <<- FALSE
+            })
+            tag(obj, ".handlers") <- handlers          
+          })
+
+## ## There is an issue here. When we set values via [<- the handlers are gone!
+## setMethod(".addhandlerclicked",
+##           signature(toolkit="guiWidgetsToolkitRGtk2",obj="gRadioRGtk"),
+##           function(obj, toolkit, handler, action=NULL, ...) {
+
+##             radiogp <- getWidget(obj)
+##             btns <- rev(radiogp$GetGroup())
+            
+##             IDs = sapply(btns, function(x) {
+##               gtktry(connectSignal(x,
+##                                    signal="toggled",
+##                                    f=function(h,w,...) {
+##                                      ## only call handler for change to active
+##                                      ## not just toggle
+##                                      if(w$GetActive())
+##                                        handler(h,w,...)
+##                                    },
+##                                    data=list(obj=obj, action=action,...),
+##                                    user.data.first = TRUE,
+##                                    after = FALSE), silent=FALSE)
+##             })
+            
+##             handler.ID = tag(obj, "handler.id")
+##             if(is.null(handler.ID))
+##               handler.ID =list()
+##             for(i in 1:length(IDs))
+##               handler.ID[[length(handler.ID)+1]] = IDs[[i]]
+##             tag(obj, "handler.id", replace=FALSE) <- handler.ID
+            
+##             invisible(IDs)
+##           })
  
 
